@@ -1,5 +1,6 @@
 from django.db.models.signals import post_save
 from django.conf import settings
+from django.db.models import ObjectDoesNotExist
 
 from pybb.subscription import notify_topic_subscribers
 from django.contrib.auth.models import User, Permission
@@ -19,9 +20,9 @@ def user_saved(instance, created, **kwargs):
     if not created:
         return
     try:
-        add_post_permission = Permission.objects.get(codename='add_post', content_type__name='Post')
-        add_topic_permission = Permission.objects.get(codename='add_topic', content_type__name='Topic')
-    except Permission.DoesNotExist:
+        add_post_permission = Permission.objects.get_by_natural_key('add_post', 'pybb', 'post')
+        add_topic_permission = Permission.objects.get_by_natural_key('add_topic', 'pybb', 'topic')
+    except ObjectDoesNotExist:
         return
     instance.user_permissions.add(add_post_permission, add_topic_permission)
     instance.save()
